@@ -2,18 +2,15 @@ from __future__ import annotations
 
 import os
 import re
-import warnings
 
 from contextlib import suppress
 from pathlib import Path
 from typing import TYPE_CHECKING
-from typing import Iterable
 from typing import TypeVar
 
 from packaging.utils import canonicalize_name
 
 from poetry.core.constraints.generic import parse_constraint as parse_generic_constraint
-from poetry.core.constraints.version import VersionRangeConstraint
 from poetry.core.constraints.version import parse_constraint
 from poetry.core.packages.dependency_group import MAIN_GROUP
 from poetry.core.packages.specification import PackageSpecification
@@ -24,6 +21,9 @@ from poetry.core.version.markers import parse_marker
 
 
 if TYPE_CHECKING:
+    from collections.abc import Iterable
+    from collections.abc import Sequence
+
     from packaging.utils import NormalizedName
 
     from poetry.core.constraints.version import VersionConstraint
@@ -41,7 +41,7 @@ class Dependency(PackageSpecification):
         constraint: str | VersionConstraint,
         optional: bool = False,
         groups: Iterable[str] | None = None,
-        allows_prereleases: bool = False,
+        allows_prereleases: bool | None = None,
         extras: Iterable[str] | None = None,
         source_type: str | None = None,
         source_url: str | None = None,
@@ -61,6 +61,9 @@ class Dependency(PackageSpecification):
             features=extras,
         )
 
+        # Attributes must be immutable for clone() to be safe!
+        # (For performance reasons, clone only creates a copy instead of a deep copy).
+
         self._constraint: VersionConstraint
         self._pretty_constraint: str
         self.constraint = constraint  # type: ignore[assignment]
@@ -71,24 +74,15 @@ class Dependency(PackageSpecification):
             groups = [MAIN_GROUP]
 
         self._groups = frozenset(groups)
-
-        if (
-            isinstance(self._constraint, VersionRangeConstraint)
-            and self._constraint.min
-        ):
-            allows_prereleases = (
-                allows_prereleases or self._constraint.min.is_unstable()
-            )
-
         self._allows_prereleases = allows_prereleases
+        # "_develop" is only required for enriching [project] dependencies
+        self._develop = False
 
         self._python_versions = "*"
         self._python_constraint = parse_constraint("*")
-        self._transitive_python_versions: str | None = None
-        self._transitive_python_constraint: VersionConstraint | None = None
         self._transitive_marker: BaseMarker | None = None
 
-        self._in_extras: list[NormalizedName] = []
+        self._in_extras: Sequence[NormalizedName] = []
 
         self._activated = not self._optional
 
@@ -113,15 +107,6 @@ class Dependency(PackageSpecification):
 
         self._pretty_constraint = str(constraint)
 
-    def set_constraint(self, constraint: str | VersionConstraint) -> None:
-        warnings.warn(
-            "Calling method 'set_constraint' is deprecated and will be removed. "
-            "It has been replaced by the property 'constraint' for consistency.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        self.constraint = constraint  # type: ignore[assignment]
-
     @property
     def pretty_constraint(self) -> str:
         return self._pretty_constraint
@@ -149,18 +134,6 @@ class Dependency(PackageSpecification):
                 )
             )
 
-    @property
-    def transitive_python_versions(self) -> str:
-        if self._transitive_python_versions is None:
-            return self._python_versions
-
-        return self._transitive_python_versions
-
-    @transitive_python_versions.setter
-    def transitive_python_versions(self, value: str) -> None:
-        self._transitive_python_versions = value
-        self._transitive_python_constraint = parse_constraint(value)
-
     @property
     def marker(self) -> BaseMarker:
         return self._marker
@@ -183,9 +156,18 @@ class Dependency(PackageSpecification):
             # If we have extras, the dependency is optional
             self.deactivate()
 
+            new_in_extras = []
             for or_ in markers["extra"]:
-                for _, extra in or_:
-                    self.in_extras.append(canonicalize_name(extra))
+                for op, extra in or_:
+                    if op == "==":
+                        new_in_extras.append(canonicalize_name(extra))
+                    elif op == "" and ("||" in extra or "," in extra):
+                        sep = "||" if "||" in extra else ","
+                        extra_values = [e.strip() for e in extra.split(sep)]
+                        for _extra in extra_values:
+                            if not _extra.startswith("!="):
+                                new_in_extras.append(canonicalize_name(_extra))
+            self._in_extras = [*self._in_extras, *new_in_extras]
 
         # Recalculate python versions.
         self._python_versions = "*"
@@ -212,20 +194,13 @@ class Dependency(PackageSpecification):
     def python_constraint(self) -> VersionConstraint:
         return self._python_constraint
 
-    @property
-    def transitive_python_constraint(self) -> VersionConstraint:
-        if self._transitive_python_constraint is None:
-            return self._python_constraint
-
-        return self._transitive_python_constraint
-
     @property
     def extras(self) -> frozenset[NormalizedName]:
         # extras activated in a dependency is the same as features
         return self._features
 
     @property
-    def in_extras(self) -> list[NormalizedName]:
+    def in_extras(self) -> Sequence[NormalizedName]:
         return self._in_extras
 
     @property
@@ -233,22 +208,18 @@ class Dependency(PackageSpecification):
         from poetry.core.constraints.version import Version
         from poetry.core.constraints.version import VersionUnion
 
-        requirement = self.pretty_name
-
-        if self.extras:
-            extras = ",".join(sorted(self.extras))
-            requirement += f"[{extras}]"
+        requirement = self.complete_pretty_name
 
         constraint = self.constraint
         if isinstance(constraint, VersionUnion):
             if (
-                constraint.excludes_single_version()
-                or constraint.excludes_single_wildcard_range()
+                constraint.excludes_single_version
+                or constraint.excludes_single_wildcard_range
             ):
                 # This branch is a short-circuit logic for special cases and
                 # avoids having to split and parse constraint again. This has
                 # no functional difference with the logic in the else branch.
-                requirement += f" ({str(constraint)})"
+                requirement += f" ({constraint})"
             else:
                 constraints = ",".join(
                     str(parse_constraint(c)) for c in self.pretty_constraint.split(",")
@@ -261,7 +232,18 @@ class Dependency(PackageSpecification):
 
         return requirement
 
-    def allows_prereleases(self) -> bool:
+    @property
+    def base_pep_508_name_resolved(self) -> str:
+        return self.base_pep_508_name
+
+    def allows_prereleases(self) -> bool | None:
+        """
+        None (default): only use pre-release versions
+                        if no stable version satisfies the constraint
+        False: do not allow pre-release versions
+               even if this means there is no solution
+        True: do not distinguish between stable and pre-release versions
+        """
         return self._allows_prereleases
 
     def is_optional(self) -> bool:
@@ -282,10 +264,13 @@ class Dependency(PackageSpecification):
     def is_url(self) -> bool:
         return False
 
-    def to_pep_508(self, with_extras: bool = True) -> str:
+    def to_pep_508(self, with_extras: bool = True, *, resolved: bool = False) -> str:
         from poetry.core.packages.utils.utils import convert_markers
 
-        requirement = self.base_pep_508_name
+        if resolved:
+            requirement = self.base_pep_508_name_resolved
+        else:
+            requirement = self.base_pep_508_name
 
         markers = []
         has_extras = False
@@ -296,7 +281,7 @@ class Dependency(PackageSpecification):
 
             # we re-check for any marker here since the without extra marker might
             # return an any marker again
-            if not marker.is_empty() and not marker.is_any():
+            if not (marker.is_empty() or marker.is_any()):
                 markers.append(str(marker))
 
             has_extras = "extra" in convert_markers(marker)
@@ -349,12 +334,13 @@ class Dependency(PackageSpecification):
         cls, name: str, relative_to: Path | None = None
     ) -> Dependency:
         """
-        Resolve a PEP-508 requirement string to a `Dependency` instance. If a `relative_to`
-        path is specified, this is used as the base directory if the identified dependency is
-        of file or directory type.
+        Resolve a PEP-508 requirement string to a `Dependency` instance. If a
+        `relative_to` path is specified, this is used as the base directory if the
+        identified dependency is of file or directory type.
         """
         from poetry.core.packages.url_dependency import URLDependency
         from poetry.core.packages.utils.link import Link
+        from poetry.core.packages.utils.utils import cached_is_dir
         from poetry.core.packages.utils.utils import is_archive_file
         from poetry.core.packages.utils.utils import is_python_project
         from poetry.core.packages.utils.utils import is_url
@@ -364,7 +350,7 @@ class Dependency(PackageSpecification):
         from poetry.core.packages.vcs_dependency import VCSDependency
         from poetry.core.utils.patterns import wheel_file_re
         from poetry.core.vcs.git import ParsedUrl
-        from poetry.core.version.requirements import Requirement
+        from poetry.core.version.requirements import parse_requirement
 
         # Removing comments
         parts = name.split(" #", 1)
@@ -374,7 +360,7 @@ class Dependency(PackageSpecification):
             if " ;" in rest:
                 name += " ;" + rest.split(" ;", 1)[1]
 
-        req = Requirement(name)
+        req = parse_requirement(name)
 
         name = req.name
         link = None
@@ -384,13 +370,12 @@ class Dependency(PackageSpecification):
         elif req.url:
             link = Link(req.url)
         else:
-            path_str = os.path.normpath(os.path.abspath(name))
+            path_str = os.path.normpath(os.path.abspath(name))  # noqa: PTH100
             p, extras = strip_extras(path_str)
-            if os.path.isdir(p) and (os.path.sep in name or name.startswith(".")):
+            if cached_is_dir(p) and (os.path.sep in name or name.startswith(".")):
                 if not is_python_project(Path(name)):
                     raise ValueError(
-                        f"Directory {name!r} is not installable. File 'setup.[py|cfg]' "
-                        "not found."
+                        f"Directory {name!r} is not installable. Not a Python project."
                     )
                 link = Link(path_to_url(p))
             elif is_archive_file(p):
@@ -433,7 +418,7 @@ class Dependency(PackageSpecification):
                 dep = VCSDependency(
                     name, "git", link.url_without_fragment, extras=req.extras
                 )
-            elif link.scheme in ["http", "https"]:
+            elif link.scheme in ("http", "https"):
                 dep = URLDependency(
                     name,
                     link.url_without_fragment,
@@ -444,7 +429,11 @@ class Dependency(PackageSpecification):
                 # handle RFC 8089 references
                 path = url_to_path(req.url)
                 dep = _make_file_or_dir_dep(
-                    name=name, path=path, base=relative_to, extras=req.extras
+                    name=name,
+                    path=path,
+                    base=relative_to,
+                    subdirectory=link.subdirectory_fragment,
+                    extras=req.extras,
                 )
             else:
                 with suppress(ValueError):
@@ -463,10 +452,7 @@ class Dependency(PackageSpecification):
                 dep._constraint = parse_constraint(version)
         else:
             constraint: VersionConstraint | str
-            if req.pretty_constraint:
-                constraint = req.constraint
-            else:
-                constraint = "*"
+            constraint = req.constraint if req.pretty_constraint else "*"
             dep = Dependency(name, constraint, extras=req.extras)
 
         if req.marker:
@@ -501,30 +487,43 @@ class Dependency(PackageSpecification):
         return self.base_pep_508_name
 
     def __repr__(self) -> str:
-        return f"<{self.__class__.__name__} {str(self)}>"
+        return f"<{self.__class__.__name__} {self}>"
 
 
 def _make_file_or_dir_dep(
     name: str,
     path: Path,
     base: Path | None = None,
-    extras: list[str] | None = None,
+    subdirectory: str | None = None,
+    extras: Iterable[str] | None = None,
 ) -> FileDependency | DirectoryDependency | None:
     """
-    Helper function to create a file or directoru dependency with the given arguments. If
-    path is not a file or directory that exists, `None` is returned.
+    Helper function to create a file or directoru dependency with the given arguments.
+
+    If path is not a file or directory that exists, a guess is made based on the suffix
+    of the given path. This is done to prevent dependendencies from being parsed as normal
+    dependencies. This allows for downstream error handling.
+
+    See also: poetry#10068
     """
     from poetry.core.packages.directory_dependency import DirectoryDependency
     from poetry.core.packages.file_dependency import FileDependency
 
     _path = path
+
     if not path.is_absolute() and base:
         # a base path was specified, so we should respect that
         _path = Path(base) / path
 
-    if _path.is_file():
-        return FileDependency(name, path, base=base, extras=extras)
-    elif _path.is_dir():
-        return DirectoryDependency(name, path, base=base, extras=extras)
+    # we check if it is a file (if it exists) or rely on suffix to guess
+    is_file = _path.is_file() if _path.exists() else path.suffix != ""
 
-    return None
+    if is_file:
+        return FileDependency(
+            name, path, base=base, directory=subdirectory, extras=extras
+        )
+
+    if subdirectory:
+        path = path / subdirectory
+
+    return DirectoryDependency(name, path, base=base, extras=extras)
