@@ -171,6 +171,22 @@ def test_single_marker_intersect() -> None:
     assert str(intersection) == 'python_version >= "3.4" and python_version < "3.6"'
 
 
+@pytest.mark.parametrize(
+    ("marker1", "marker2"),
+    [
+        ('sys_platform == "win32"', 'sys_platform == "linux"'),
+        ('sys_platform != "win32"', 'sys_platform == "win32"'),
+        ('python_version >= "3.6"', 'python_version < "3.6"'),
+        ('python_version == "3.6"', 'python_version == "3.7"'),
+        ('python_version > "3.6"', 'python_version <= "3.6"'),
+        ('python_version >= "3.6"', 'python_version < "3.6"'),
+        ('python_version > "3.6"', 'python_version < "3.7"'),
+    ],
+)
+def test_single_marker_intersect_is_empty(marker1: str, marker2: str) -> None:
+    assert parse_marker(marker1).intersect(parse_marker(marker2)).is_empty()
+
+
 @pytest.mark.parametrize(
     ("marker1", "marker2", "expected"),
     [
@@ -296,6 +312,16 @@ def test_single_marker_not_in_python_intersection() -> None:
             'extra == "b" and extra == "a"',
             'extra == "a" and extra == "b"',
         ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "a" and extra == "b" and extra == "c"',
+            'extra == "a" and extra == "b" and extra == "c"',
+        ),
+        (
+            'extra == "a" and extra == "b" and extra == "c" and extra == "d"',
+            'extra == "a" and extra == "b"',
+            'extra == "a" and extra == "b" and extra == "c" and extra == "d"',
+        ),
         (
             'extra == "a" and extra == "b"',
             'extra == "c" and extra != "d"',
@@ -341,6 +367,16 @@ def test_single_marker_not_in_python_intersection() -> None:
             'extra == "b" or extra == "a"',
             'extra == "a" or extra == "b"',
         ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra == "b" or extra == "c"',
+            'extra == "a" or extra == "b"',
+        ),
+        (
+            'extra == "a" or extra == "b" or extra == "c" or extra == "d"',
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra == "b"',
+        ),
         (
             'extra == "a" or extra == "b"',
             'extra == "a" or extra != "c"',
@@ -357,6 +393,16 @@ def test_single_marker_not_in_python_intersection() -> None:
             'extra == "a" or extra == "c"',
             'extra == "c" and extra != "a" and extra != "b"',
         ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "a" or extra == "b"',
+            'extra == "a" and extra == "b"',
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "a" and extra == "b"',
+            'extra == "a" and extra == "b"',
+        ),
     ],
 )
 def test_single_marker_intersect_extras(
@@ -586,6 +632,16 @@ def test_single_marker_union_with_inverse() -> None:
             'extra == "a" and extra == "b"',
             'extra == "a" and extra == "b"',
         ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "a" and extra == "b" and extra == "c"',
+            'extra == "a" and extra == "b"',
+        ),
+        (
+            'extra == "a" and extra == "b" and extra == "c" and extra == "d"',
+            'extra == "a" and extra == "b"',
+            'extra == "a" and extra == "b"',
+        ),
         (
             'extra == "a" and extra == "b"',
             'extra == "c" and extra != "d"',
@@ -642,6 +698,16 @@ def test_single_marker_union_with_inverse() -> None:
             'extra == "a" or extra == "b"',
             'extra == "a" or extra == "b"',
         ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra == "b" or extra == "c"',
+            'extra == "a" or extra == "b" or extra == "c"',
+        ),
+        (
+            'extra == "a" or extra == "b" or extra == "c" or extra == "d"',
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra == "b" or extra == "c" or extra == "d"',
+        ),
         (
             'extra == "a" or extra == "b"',
             'extra == "a" or extra != "c"',
@@ -658,6 +724,16 @@ def test_single_marker_union_with_inverse() -> None:
             'extra == "a" or extra == "c"',
             'extra != "a" and extra != "b" or extra == "a" or extra == "c"',
         ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "a" and extra == "c"',
+            'extra == "a" or extra == "b"',
+        ),
+        (
+            'extra == "a" and extra == "c"',
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra == "b"',
+        ),
     ],
 )
 def test_single_marker_union_extras(marker1: str, marker2: str, expected: str) -> None:
@@ -821,6 +897,96 @@ def test_multi_marker_union_multi_is_multi(
 @pytest.mark.parametrize(
     "marker1, marker2, expected",
     [
+        # Equivalent ranges
+        (
+            'python_version == "3.8" or python_version == "3.9"',
+            'python_version >= "3.8" and python_version <= "3.9"',
+            'python_version >= "3.8" and python_version < "3.10"',
+        ),
+        (
+            'python_version == "3.8" or python_version == "3.9"',
+            (
+                'python_version >= "3.8" and python_version <= "3.9"'
+                ' and sys_platform == "linux"'
+            ),
+            'python_version >= "3.8" and python_version < "3.10"',
+        ),
+        (
+            'python_version == "3.8" or python_version == "3.9"',
+            (
+                'python_version >= "3.8" and python_version <= "3.9"'
+                ' and sys_platform == "linux"'
+            ),
+            'python_version >= "3.8" and python_version < "3.10"',
+        ),
+        (
+            'python_version == "3.8" or python_version == "3.9"',
+            (
+                'python_version >= "3.8" and python_version < "3.10"'
+                ' and sys_platform == "linux"'
+            ),
+            'python_version >= "3.8" and python_version < "3.10"',
+        ),
+        (
+            'python_version == "3.8" or python_version == "3.9"',
+            (
+                'python_version > "3.7" and python_version < "3.10"'
+                ' and sys_platform == "linux"'
+            ),
+            'python_version > "3.7" and python_version < "3.10"',
+        ),
+        (
+            'python_version == "3.8" or python_version == "3.9"',
+            (
+                'python_version > "3.7" and python_version <= "3.9"'
+                ' and sys_platform == "linux"'
+            ),
+            'python_version > "3.7" and python_version < "3.10"',
+        ),
+        (
+            (
+                'python_version == "3.8" or python_version == "3.9"'
+                ' or python_version == "3.10"'
+            ),
+            (
+                'python_version >= "3.8" and python_version <= "3.10"'
+                ' and sys_platform == "linux"'
+            ),
+            'python_version >= "3.8" and python_version <= "3.10"',
+        ),
+        (
+            (
+                'python_version == "3.8" or python_version == "3.9"'
+                ' or python_version == "3.10"'
+            ),
+            (
+                'python_version >= "3.8" and python_version < "3.11"'
+                ' and sys_platform == "linux"'
+            ),
+            'python_version >= "3.8" and python_version < "3.11"',
+        ),
+        (
+            (
+                'python_version == "3.8" or python_version == "3.9"'
+                ' or python_version == "3.10"'
+            ),
+            (
+                'python_version > "3.7" and python_version <= "3.10"'
+                ' and sys_platform == "linux"'
+            ),
+            'python_version > "3.7" and python_version <= "3.10"',
+        ),
+        (
+            (
+                'python_version == "3.8" or python_version == "3.9"'
+                ' or python_version == "3.10"'
+            ),
+            (
+                'python_version > "3.7" and python_version < "3.11"'
+                ' and sys_platform == "linux"'
+            ),
+            'python_version > "3.7" and python_version < "3.11"',
+        ),
         # Ranges with same start
         (
             'python_version >= "3.6" and python_full_version < "3.6.2"',
@@ -832,17 +998,17 @@ def test_multi_marker_union_multi_is_multi(
             'python_version >= "3.6" and python_version < "3.8"',
             'python_version >= "3.6" and python_version < "3.8"',
         ),
-        (
-            'python_version > "3.6" and python_full_version < "3.6.2"',
-            'python_version > "3.6" and python_version < "3.7"',
-            'python_version > "3.6" and python_version < "3.7"',
-        ),
         (
             'python_version > "3.6" and python_full_version < "3.7.2"',
             'python_version > "3.6" and python_version < "3.8"',
             'python_version > "3.6" and python_version < "3.8"',
         ),
-        # Ranges meet exactly
+        (
+            'python_version > "3.7"',
+            'python_version >= "3.8" and sys_platform == "linux"',
+            'python_version > "3.7"',
+        ),
+        # Ranges meet exactly (adjacent ranges)
         (
             'python_version >= "3.6" and python_full_version < "3.6.2"',
             'python_full_version >= "3.6.2" and python_version < "3.7"',
@@ -863,6 +1029,21 @@ def test_multi_marker_union_multi_is_multi(
             'python_full_version > "3.6.2" and python_version < "3.8"',
             'python_version >= "3.6" and python_version < "3.8"',
         ),
+        (
+            'python_version >= "3.8" and python_version < "3.9"',
+            'python_version >= "3.9" and python_version < "3.11"',
+            'python_version >= "3.8" and python_version < "3.11"',
+        ),
+        (
+            'python_version >= "3.8" and python_version < "3.9"',
+            'python_version >= "3.9" and python_version < "3.10"',
+            'python_version >= "3.8" and python_version < "3.10"',
+        ),
+        (
+            'python_version == "3.8"',
+            'python_version == "3.9"',
+            'python_version >= "3.8" and python_version < "3.10"',
+        ),
         # Ranges overlap
         (
             'python_version >= "3.6" and python_full_version <= "3.6.8"',
@@ -921,6 +1102,39 @@ def test_multi_marker_union_multi_is_multi(
             'python_version == "3.7" and implementation_name == "cpython"',
             'python_version >= "3.6" and python_version <= "3.7"',
         ),
+        # complex
+        (
+            'python_version == "3.10" and platform_system == "Linux"',
+            (
+                'python_version == "3.11" and platform_system != "Darwin"'
+                ' or platform_system != "Linux" and platform_system != "Darwin"'
+                ' and python_version <= "3.11" and python_full_version >= "3.10.0"'
+            ),
+            (
+                'python_version >= "3.10" and python_version <= "3.11"'
+                ' and platform_system != "Darwin"'
+            ),
+        ),
+        (
+            (
+                'python_version == "3.6" and platform_system == "Linux"'
+                ' and platform_machine == "aarch64" or python_version == "3.9"'
+                ' and platform_system != "Darwin" or python_version == "3.9"'
+                ' and platform_machine != "arm64" or python_version >= "3.8"'
+                ' and platform_system == "Linux" and platform_machine == "aarch64"'
+                ' and python_version < "3.10"'
+            ),
+            (
+                'python_version == "3.7" and platform_system == "Linux"'
+                ' and platform_machine == "aarch64"'
+            ),
+            (
+                'python_version >= "3.6" and python_version < "3.10"'
+                ' and platform_system == "Linux" and platform_machine == "aarch64"'
+                ' or python_version == "3.9" and platform_machine != "arm64"'
+                ' or python_version == "3.9" and platform_system != "Darwin"'
+            ),
+        ),
     ],
 )
 def test_version_ranges_collapse_on_union(
@@ -1487,7 +1701,7 @@ def test_only(marker: str, only: list[str], expected: str) -> None:
         ("", "~3.8", ""),
         ("<empty>", "~3.8", "<empty>"),
         ('sys_platform == "linux"', "~3.8", 'sys_platform == "linux"'),
-        ('python_version >= "3.8"', "~3.8", ""),
+        ('python_version == "3.8"', "~3.8", ""),
         ('python_version == "3.8"', ">=3.8.7,<3.9.0", ""),
         ('python_version == "3.8" or python_version >= "3.9"', ">=3.8.0,<4.0.0", ""),
         ('python_version == "3.8" or python_version >= "3.9"', ">=3.8.7,<4.0.0", ""),
@@ -1495,6 +1709,11 @@ def test_only(marker: str, only: list[str], expected: str) -> None:
         ('python_version > "3.8"', "~3.8", "<empty>"),
         ('python_version >= "3.8"', "~3.8", ""),
         ('python_version >= "3.9"', "~3.8", "<empty>"),
+        ('python_version == "3.9"', ">=3.9", 'python_version == "3.9"'),
+        ('python_version <= "3.9"', ">=3.9", 'python_version == "3.9"'),
+        ('python_version < "3.10"', ">=3.9", 'python_version == "3.9"'),
+        ('python_version <= "3.10"', ">=3.9", 'python_version <= "3.10"'),
+        ('python_version < "3.11"', ">=3.9", 'python_version < "3.11"'),
         ('python_full_version >= "3.8.0"', "~3.8", ""),
         ('python_full_version >= "3.8.1"', "~3.8", 'python_full_version >= "3.8.1"'),
         ('python_full_version < "3.8.0"', "~3.8", "<empty>"),
@@ -1519,7 +1738,7 @@ def test_only(marker: str, only: list[str], expected: str) -> None:
         (
             'python_version < "3.8" or python_version >= "3.9"',
             ">=3.7",
-            'python_version < "3.8" or python_version >= "3.9"',
+            'python_version == "3.7" or python_version >= "3.9"',
         ),
         ('python_version < "3.8" or python_version >= "3.9"', "~3.7", ""),
         (
@@ -1527,6 +1746,11 @@ def test_only(marker: str, only: list[str], expected: str) -> None:
             "<=3.10",
             'python_version < "3.8" or python_version >= "3.9"',
         ),
+        (
+            'python_version == "3.8" or sys_platform == "linux"',
+            "~3.8",
+            "",
+        ),
         (
             (
                 'python_version < "3.8"'
@@ -1539,7 +1763,7 @@ def test_only(marker: str, only: list[str], expected: str) -> None:
         (
             'python_version < "3.8" or python_version >= "3.9"',
             "~3.6 || ~3.8",
-            'python_version < "3.8"',
+            'python_version == "3.6"',
         ),
         (
             (
@@ -1547,7 +1771,7 @@ def test_only(marker: str, only: list[str], expected: str) -> None:
                 ' or python_version >= "3.9"'
             ),
             "~3.7 || ~3.9",
-            'sys_platform == "linux"',
+            "",
         ),
         (
             (
@@ -1555,17 +1779,12 @@ def test_only(marker: str, only: list[str], expected: str) -> None:
                 ' or python_version >= "3.9" or sys_platform == "win32"'
             ),
             "~3.7 || ~3.9",
-            'sys_platform == "linux" or sys_platform == "win32"',
+            "",
         ),
         (
-            'python_version == "3.8" or sys_platform == "linux" or python_version >= "3.9"',
-            ">=3.8.0,<4.0.0",
-            'sys_platform == "linux"',
-        ),
-        (
-            'python_version == "3.8" or sys_platform == "linux" or python_version >= "3.9"',
-            ">=3.8.7,<4.0.0",
-            'sys_platform == "linux"',
+            '(python_version == "3.8" or platform_system == "Linux") and sys_platform == "darwin"',
+            "~3.8",
+            'sys_platform == "darwin"',
         ),
     ],
 )
@@ -2101,8 +2320,8 @@ def test_complex_union() -> None:
     ]
     assert (
         str(union(*markers))
-        == 'python_version >= "3.6" and platform_system == "Darwin"'
-        ' and platform_machine == "arm64" or python_version >= "3.10"'
+        == 'platform_system == "Darwin" and platform_machine == "arm64"'
+        ' and python_version >= "3.6" or python_version >= "3.10"'
     )
 
 
@@ -2134,8 +2353,75 @@ def test_complex_intersection() -> None:
     ]
     assert (
         str(dnf(intersection(*markers).invert()))
-        == 'python_version >= "3.6" and platform_system == "Darwin"'
-        ' and platform_machine == "arm64" or python_version >= "3.10"'
+        == 'platform_system == "Darwin" and platform_machine == "arm64"'
+        ' and python_version >= "3.6" or python_version >= "3.10"'
+    )
+
+
+def test_complex_intersection_with_itertools_product_duplicates() -> None:
+    """
+    Real-world example from https://github.com/python-poetry/poetry/issues/10250.
+    (Only occurs if the solver takes an unfortunate path.)
+    Takes a long time without filtering duplicates from the itertools.product()
+    in cnf/dnf early.
+    """
+    m1 = parse_marker(
+        '(python_version > "3.9" or platform_system != "Windows"'
+        ' or platform_machine != "x86") and (python_version != "3.10"'
+        ' or platform_system != "Windows" or platform_python_implementation == "PyPy")'
+    )
+    m2 = parse_marker(
+        '(platform_system != "Windows" or platform_machine != "x86"'
+        ' or python_version >= "3.10" and python_version < "3.12")'
+        ' and (sys_platform != "darwin" or platform_machine != "arm64")'
+        ' and python_version <= "3.11" and (platform_system != "Windows"'
+        ' or platform_python_implementation == "PyPy" or python_version == "3.9"'
+        ' or python_version == "3.11") and python_version >= "3.9"'
+        ' and (platform_system != "Windows" or platform_python_implementation == "PyPy"'
+        ' or platform_machine != "x86" or python_version == "3.11")'
+    )
+
+    assert str(m1.intersect(m2)) == (
+        '(python_version > "3.9" or platform_system != "Windows"'
+        ' or platform_machine != "x86") and (python_version != "3.10"'
+        ' or platform_system != "Windows" or platform_python_implementation == "PyPy")'
+        ' and (platform_system != "Windows" or platform_machine != "x86"'
+        ' or python_version >= "3.10") and python_version <= "3.11"'
+        ' and (sys_platform != "darwin" or platform_machine != "arm64") and'
+        ' (platform_system != "Windows" or platform_python_implementation == "PyPy"'
+        ' or python_version == "3.9" or python_version == "3.11")'
+        ' and python_version >= "3.9" and (platform_system != "Windows"'
+        ' or platform_python_implementation == "PyPy" or platform_machine != "x86"'
+        ' or python_version == "3.11")'
+    )
+
+
+def test_complex_union_is_deterministic() -> None:
+    """
+    This test might fail sporadically if marker operations are not deterministic!
+    """
+    m1 = parse_marker(
+        'sys_platform != "darwin" and python_version >= "3.12"'
+        ' and platform_system != "Emscripten" and (python_version < "4.0"'
+        ' and sys_platform == "linux" and extra == "stretch"'
+        ' or platform_system == "Windows" or extra == "test"'
+        ' and sys_platform == "win32")'
+    )
+    m2 = parse_marker(
+        'sys_platform == "linux" and python_version >= "3.12"'
+        ' and platform_system == "Emscripten" and python_version < "4.0"'
+        ' and extra == "stretch" or sys_platform == "win32"'
+        ' and python_version >= "3.12" and platform_system == "Emscripten"'
+        ' and extra == "test"'
+    )
+    assert str(m1.union(m2)) == (
+        'python_version >= "3.12" and platform_system == "Windows"'
+        ' and sys_platform != "darwin" or sys_platform == "linux"'
+        ' and python_version >= "3.12" and python_version < "4.0"'
+        ' and extra == "stretch" or sys_platform == "win32"'
+        ' and python_version >= "3.12" and extra == "test" or (sys_platform == "linux"'
+        ' or sys_platform == "win32") and python_version >= "3.12"'
+        ' and extra == "stretch" and extra == "test" and python_version < "4.0"'
     )
 
 
@@ -2196,28 +2482,56 @@ def test_intersection_avoids_combinatorial_explosion() -> None:
     )
 
 
-def test_intersection_no_endless_recursion() -> None:
-    m1 = parse_marker(
-        '(python_version < "3.9" or extra != "bigquery" and extra != "parquet"'
-        ' and extra != "motherduck" and extra != "athena" and extra != "synapse"'
-        ' and extra != "clickhouse" and extra != "dremio" and extra != "lancedb"'
-        ' and extra != "deltalake" and extra != "pyiceberg"'
-        ' and python_version < "3.13") and extra != "postgres" and extra != "redshift"'
-        ' and extra != "postgis"'
-    )
-    m2 = parse_marker(
-        'python_version > "3.12" and python_version < "3.13" or extra != "databricks"'
-    )
-    expected = (
-        '(python_version < "3.9" or extra != "bigquery" and extra != "parquet"'
-        ' and extra != "motherduck" and extra != "athena" and extra != "synapse"'
-        ' and extra != "clickhouse" and extra != "dremio" and extra != "lancedb"'
-        ' and extra != "deltalake" and extra != "pyiceberg")'
-        ' and python_version < "3.13" and extra != "postgres" and extra != "redshift"'
-        ' and extra != "postgis" and (python_version > "3.12"'
-        ' and python_version < "3.13" or extra != "databricks")'
-    )
-    assert str(m1.intersect(m2)) == expected
+@pytest.mark.parametrize(
+    ("marker1", "marker2", "expected"),
+    [
+        (
+            (
+                '(python_version < "3.9" or extra != "bigquery" and extra != "parquet"'
+                ' and extra != "motherduck" and extra != "athena"'
+                ' and extra != "synapse" and extra != "clickhouse"'
+                ' and extra != "dremio" and extra != "lancedb"'
+                ' and extra != "deltalake" and extra != "pyiceberg"'
+                ' and python_version < "3.13") and extra != "postgres"'
+                ' and extra != "redshift" and extra != "postgis"'
+            ),
+            (
+                'python_version >= "3.12" and python_version < "3.13"'
+                ' or extra != "databricks"'
+            ),
+            (
+                '(python_version < "3.9" or extra != "bigquery" and extra != "parquet"'
+                ' and extra != "motherduck" and extra != "athena"'
+                ' and extra != "synapse" and extra != "clickhouse"'
+                ' and extra != "dremio" and extra != "lancedb"'
+                ' and extra != "deltalake" and extra != "pyiceberg")'
+                ' and python_version < "3.13" and extra != "postgres"'
+                ' and extra != "redshift" and extra != "postgis"'
+                ' and (python_version == "3.12" or extra != "databricks")'
+            ),
+        ),
+        (
+            (
+                'python_version < "3.7" and (python_version < "3.6"'
+                ' or platform_system != "Darwin" or platform_machine != "arm64")'
+            ),
+            (
+                '(python_version < "3.6" or platform_system != "Linux"'
+                ' or platform_machine != "aarch64") and python_version < "3.9"'
+            ),
+            (
+                'python_version < "3.7" and (python_version < "3.6"'
+                ' or platform_system != "Darwin" or platform_machine != "arm64")'
+                ' and (python_version < "3.6" or platform_system != "Linux"'
+                ' or platform_machine != "aarch64")'
+            ),
+        ),
+    ],
+)
+def test_intersection_no_endless_recursion(
+    marker1: str, marker2: str, expected: str
+) -> None:
+    assert str(parse_marker(marker1).intersect(parse_marker(marker2))) == expected
 
 
 @pytest.mark.parametrize(
